# Form implementation generated from reading ui file 'videosettings.ui'
#
# Created by: PyQt6 UI code generator 6.2.0.dev2107180056
#
# WARNING: Any manual changes made to this file will be lost when pyuic6 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt6 import QtCore, QtGui, QtWidgets


class Ui_VideoSettingsUi(object):
    def setupUi(self, VideoSettingsUi):
        VideoSettingsUi.setObjectName("VideoSettingsUi")
        VideoSettingsUi.resize(686, 499)
        self.gridLayout_3 = QtWidgets.QGridLayout(VideoSettingsUi)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.buttonBox = QtWidgets.QDialogButtonBox(VideoSettingsUi)
        self.buttonBox.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.StandardButton.Cancel|QtWidgets.QDialogButtonBox.StandardButton.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout_3.addWidget(self.buttonBox, 4, 1, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(VideoSettingsUi)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.videoResolutionBox = QtWidgets.QComboBox(self.groupBox_2)
        self.videoResolutionBox.setObjectName("videoResolutionBox")
        self.gridLayout_2.addWidget(self.videoResolutionBox, 1, 0, 1, 2)
        self.label_6 = QtWidgets.QLabel(self.groupBox_2)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 4, 0, 1, 2)
        self.videoFramerateBox = QtWidgets.QComboBox(self.groupBox_2)
        self.videoFramerateBox.setObjectName("videoFramerateBox")
        self.gridLayout_2.addWidget(self.videoFramerateBox, 3, 0, 1, 2)
        self.label_9 = QtWidgets.QLabel(self.groupBox_2)
        self.label_9.setObjectName("label_9")
        self.gridLayout_2.addWidget(self.label_9, 2, 0, 1, 2)
        self.label_8 = QtWidgets.QLabel(self.groupBox_2)
        self.label_8.setObjectName("label_8")
        self.gridLayout_2.addWidget(self.label_8, 0, 0, 1, 2)
        self.videoCodecBox = QtWidgets.QComboBox(self.groupBox_2)
        self.videoCodecBox.setObjectName("videoCodecBox")
        self.gridLayout_2.addWidget(self.videoCodecBox, 5, 0, 1, 2)
        self.gridLayout_3.addWidget(self.groupBox_2, 2, 1, 1, 1)
        self.widget = QtWidgets.QWidget(VideoSettingsUi)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Policy.Preferred, QtWidgets.QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setObjectName("widget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.groupBox_3 = QtWidgets.QGroupBox(self.widget)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_2 = QtWidgets.QLabel(self.groupBox_3)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.audioCodecBox = QtWidgets.QComboBox(self.groupBox_3)
        self.audioCodecBox.setObjectName("audioCodecBox")
        self.verticalLayout_2.addWidget(self.audioCodecBox)
        self.label_5 = QtWidgets.QLabel(self.groupBox_3)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_2.addWidget(self.label_5)
        self.audioSampleRateBox = QtWidgets.QSpinBox(self.groupBox_3)
        self.audioSampleRateBox.setObjectName("audioSampleRateBox")
        self.verticalLayout_2.addWidget(self.audioSampleRateBox)
        self.verticalLayout_3.addWidget(self.groupBox_3)
        self.groupBox = QtWidgets.QGroupBox(self.widget)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.verticalLayout.addWidget(self.label_3)
        self.qualitySlider = QtWidgets.QSlider(self.groupBox)
        self.qualitySlider.setMaximum(4)
        self.qualitySlider.setOrientation(QtCore.Qt.Orientation.Horizontal)
        self.qualitySlider.setObjectName("qualitySlider")
        self.verticalLayout.addWidget(self.qualitySlider)
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.verticalLayout.addWidget(self.label_4)
        self.containerFormatBox = QtWidgets.QComboBox(self.groupBox)
        self.containerFormatBox.setObjectName("containerFormatBox")
        self.verticalLayout.addWidget(self.containerFormatBox)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.gridLayout_3.addWidget(self.widget, 2, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Policy.Minimum, QtWidgets.QSizePolicy.Policy.Expanding)
        self.gridLayout_3.addItem(spacerItem, 3, 0, 1, 1)

        self.retranslateUi(VideoSettingsUi)
        self.buttonBox.accepted.connect(VideoSettingsUi.accept)
        self.buttonBox.rejected.connect(VideoSettingsUi.reject)
        QtCore.QMetaObject.connectSlotsByName(VideoSettingsUi)

    def retranslateUi(self, VideoSettingsUi):
        _translate = QtCore.QCoreApplication.translate
        VideoSettingsUi.setWindowTitle(_translate("VideoSettingsUi", "Video Settings"))
        self.groupBox_2.setTitle(_translate("VideoSettingsUi", "Video"))
        self.label_6.setText(_translate("VideoSettingsUi", "Video Codec:"))
        self.label_9.setText(_translate("VideoSettingsUi", "Framerate:"))
        self.label_8.setText(_translate("VideoSettingsUi", "Resolution:"))
        self.groupBox_3.setTitle(_translate("VideoSettingsUi", "Audio"))
        self.label_2.setText(_translate("VideoSettingsUi", "Audio Codec:"))
        self.label_5.setText(_translate("VideoSettingsUi", "Sample Rate:"))
        self.label_3.setText(_translate("VideoSettingsUi", "Quality:"))
        self.label_4.setText(_translate("VideoSettingsUi", "File Format:"))
